/*
 * @author Stefan Hepp
 * @date 16.08.2006
 * @version $Id: CsvImportExample.java,v 1.8 2006-09-06 11:00:15 stefan Exp $
 */
package com.gentics.portalnode.sdk.cmdline;

import com.gentics.api.lib.datasource.WriteableDatasource;
import com.gentics.api.lib.resolving.Changeable;
import com.gentics.api.portalnode.action.PluggableActionInvoker;
import com.gentics.api.portalnode.action.PluggableActionRequest;
import com.gentics.api.portalnode.action.PluggableActionResponse;
import com.gentics.api.portalnode.connector.PortalConnectorFactory;

import java.io.File;
import java.io.FileInputStream;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.Collections;
import java.util.Properties;

/**
 * This is a simple implementation of a csv file import.
 * The main program reads a file named csvdata.csv from the working dir and imports it.
 * The datasource is initialized with the settings from the jdbc-handle.properties file.
 */
public class CsvImportExample {
    private static final String PROPERTY_FILE = "jdbc-handle.properties";

    public static void main(String[] args) {

        try {
            
            // set the filename and the objecttype to import
            String filename = "csvdata.csv";
            String objtype = "1001";
            
            // get the file from the same package as this class. This will not work
            // if the files are within a jar file.
            File csvfile = new File(CsvImportExample.class.getResource(filename).getFile());
            String fileprefix = csvfile.getParent();

            // needed for cache configfile loading
            System.setProperty("com.gentics.portalnode.confpath", fileprefix);

            // get the datasource
            WriteableDatasource ds = getDatasource();

            // create the pluggable action
            PluggableActionRequest request = PluggableActionInvoker.createRequestObject();
            PluggableActionResponse response = PluggableActionInvoker.createResponseObject();

            // get test content from the csv file
            InputStream is = new FileInputStream(csvfile);
            Reader reader = new InputStreamReader(is, "ISO-8859-1");

            // configure the action and execute it
            request.setParameter("csv", reader);
            request.setParameter("objecttype", objtype);
            request.setParameter("delimiter", "\"");
            request.setParameter("separator", ";");
            request.setParameter("skiplines", "");
            request.setParameter("datasource", ds);
            request.setParameter("fileprefix", fileprefix);
            boolean success = PluggableActionInvoker.invokeAction("CSVDataImportAction", request, response);

            // print out response
            if ( success ) {
                System.out.println("Import was successful.");
            } else {
                System.out.println("Import had errors.");
            }
            System.out.println("Feedback was: " + response.getFeedbackMessage());

            // list all new ids, if set. Delete all new objects if the import had errors
            String[] newIds = (String[]) response.getParameter("createdids");
            if ( newIds != null && newIds.length > 0 ) {
                System.out.println("Objects have been created!");
                
                for (int i = 0; i < newIds.length; i++) {
                    String newId = newIds[i];
                    System.out.println("New Id: " + newId);

                    // simple delete impl to delete all imported objects, if the import had errors.
                    if ( !success ) {
                        System.out.println("Deleting new object..");
                        Changeable newObj = PortalConnectorFactory.getChangeableContentObject( newId, ds );
                        ds.delete(Collections.singleton(newObj));
                    }
                }
                System.out.println();
            }

            // print out the log messages
            String logMsgs = (String) response.getParameter("logmessages");
            if ( logMsgs != null ) {
                System.out.println("Log-messages were:");
                System.out.println(logMsgs);
            }

        } catch (Exception e) {
            // basic error handling ..
            e.printStackTrace();
        }

    }

    /**
     * Creates the Writeable Datasource.
     * @return datasource
     * @throws Exception
     */
    protected static WriteableDatasource getDatasource() throws Exception {
        Properties handleProperties = new Properties();
        // Load the properties for the datasource connection from the
        // filesystem.
        handleProperties.load(CsvImportExample.class
                .getResourceAsStream(PROPERTY_FILE));

        return PortalConnectorFactory.createWriteableDatasource(handleProperties);
    }

}
